#ifndef JUCE_APPCONFIG_H
#define JUCE_APPCONFIG_H


#include "juce/modules/juce_audio_basics/juce_audio_basics.h"
#include "juce/modules/juce_audio_processors/juce_audio_processors.h"
#include "juce/modules/juce_core/juce_core.h"
#include "juce/modules/juce_data_structures/juce_data_structures.h"
#include "juce/modules/juce_events/juce_events.h"
#include "juce/modules/juce_graphics/juce_graphics.h"
#include "juce/modules/juce_gui_basics/juce_gui_basics.h"
#include "juce/modules/juce_gui_extra/juce_gui_extra.h"


#endif
